@@ -64,6 +64,7 @@
 #include <time.h>
 #include <cmath>
 #include <algorithm>
+#include <random>
 
 #include "directoryloader.h"
 #include "playlistitem.h"
@@ -74,7 +75,6 @@
 #include "collectionlist.h"
 #include "filerenamer.h"
 #include "actioncollection.h"
-#include "tracksequencemanager.h"
 #include "juktag.h"
 #include "upcomingplaylist.h"
 #include "deletedialog.h"
@@ -123,10 +123,24 @@ Playlist::Playlist(
     : QTreeWidget(collection->playlistStack())
     , m_collection(collection)
     , m_playlistName(name)
+    , m_refillDebounce(new QTimer(this))
     , m_fetcher(new WebImageFetcher(this))
 {
     setup(extraCols);
 
+    // The timer soaks up repeated events that may cause the random play list
+    // to be regenerated repeatedly.
+    m_refillDebounce->setInterval(100);
+    m_refillDebounce->setSingleShot(true);
+    connect(m_refillDebounce, &QTimer::timeout,
+            this,             &Playlist::refillRandomList);
+
+    // Any of the random-related actions being triggered will cause the parent
+    // group to emit the triggered signal.
+    QActionGroup *randomGroup = action("disableRandomPlay")->actionGroup();
+    connect(randomGroup,      &QActionGroup::triggered,
+            m_refillDebounce, QOverload<>::of(&QTimer::start));
+
     // Some subclasses need to do even more handling but will remember to
     // call setupPlaylist
     if(!delaySetup) {
@@ -175,6 +189,7 @@ Playlist::~Playlist()
     // make clear that it's intentional that those subclassed versions don't
     // get called (because we can't call them)
 
+    m_randomSequence.clear();
     Playlist::clearItems(Playlist::items());
 
     if(!m_shuttingDown)
@@ -196,31 +211,85 @@ FileHandle Playlist::currentFile() const
 
 void Playlist::playFirst()
 {
-    TrackSequenceManager::instance()->setNextItem(static_cast<PlaylistItem *>(
-        *QTreeWidgetItemIterator(const_cast<Playlist *>(this), QTreeWidgetItemIterator::NotHidden)));
-    action("forward")->trigger();
+    QTreeWidgetItemIterator listIt(const_cast<Playlist *>(this), QTreeWidgetItemIterator::NotHidden);
+    beginPlayingItem(static_cast<PlaylistItem *>(*listIt));
+    refillRandomList();
 }
 
 void Playlist::playNextAlbum()
 {
-    PlaylistItem *current = TrackSequenceManager::instance()->currentItem();
-    if(!current)
-        return; // No next album if we're not already playing.
+    const auto &item = playingItem();
+    if(!item || !action("albumRandomPlay")->isChecked()) {
+        playNext();
+        return;
+    }
 
-    QString currentAlbum = current->file().tag()->album();
-    current = TrackSequenceManager::instance()->nextItem();
+    const auto currentAlbum = item->file().tag()->album();
+    const auto nextAlbumTrack = std::find_if(m_randomSequence.begin(), m_randomSequence.end(),
+            [currentAlbum](const PlaylistItem *item) {
+                return item->file().tag()->album() != currentAlbum;
+            });
 
-    while(current && current->file().tag()->album() == currentAlbum)
-        current = TrackSequenceManager::instance()->nextItem();
-
-    TrackSequenceManager::instance()->setNextItem(current);
-    action("forward")->trigger();
+    if(nextAlbumTrack == m_randomSequence.end()) {
+        // We were on the last album, playNext will handle looping if we should loop
+        m_randomSequence.clear();
+        playNext();
+    }
+    else {
+        m_randomSequence.erase(m_randomSequence.begin(), nextAlbumTrack);
+        beginPlayingItem(*nextAlbumTrack);
+    }
 }
 
 void Playlist::playNext()
 {
-    TrackSequenceManager::instance()->setCurrentPlaylist(this);
-    setPlaying(TrackSequenceManager::instance()->nextItem());
+    PlaylistItem *next = nullptr;
+    auto nowPlaying = playingItem();
+    bool doLoop = action("loopPlaylist")->isChecked();
+
+    // Treat an item from a different playlist as if we were being asked to
+    // play from a stop
+    if(nowPlaying && nowPlaying->playlist() != this) {
+        nowPlaying = nullptr;
+    }
+
+    if(action("disableRandomPlay")->isChecked()) {
+        QTreeWidgetItemIterator listIt = nowPlaying
+            ? QTreeWidgetItemIterator(nowPlaying, QTreeWidgetItemIterator::NotHidden)
+            : QTreeWidgetItemIterator(this,       QTreeWidgetItemIterator::NotHidden);
+
+        if(*listIt && nowPlaying) {
+            ++listIt;
+        }
+
+        next = static_cast<PlaylistItem *>(*listIt);
+
+        if(!next && doLoop) {
+            playFirst();
+            return;
+        }
+    }
+    else {
+        // The two random play modes are identical here, the difference is in how the
+        // randomized sequence is generated by refillRandomList
+
+        if(m_randomSequence.isEmpty() && (doLoop || !nowPlaying)) {
+            refillRandomList();
+
+            // Don't play the same track twice in a row even if it can
+            // "randomly" happen
+            if(m_randomSequence.front() == nowPlaying) {
+                std::swap(m_randomSequence.front(), m_randomSequence.back());
+            }
+        }
+
+        if(!m_randomSequence.isEmpty()) {
+            next = m_randomSequence.takeFirst();
+        }
+    }
+
+    // Will stop playback if next is still null
+    beginPlayingItem(next);
 }
 
 void Playlist::stop()
@@ -245,13 +314,11 @@ void Playlist::playPrevious()
     }
     else {
         m_history.clear();
-        previous = TrackSequenceManager::instance()->previousItem();
+        QTreeWidgetItemIterator listIt(playingItem(), QTreeWidgetItemIterator::NotHidden);
+        previous = static_cast<PlaylistItem *>(*--listIt);
     }
 
-    if(!previous)
-        previous = static_cast<PlaylistItem *>(playingItem()->itemAbove());
-
-    setPlaying(previous, false);
+    beginPlayingItem(previous);
 }
 
 void Playlist::setName(const QString &n)
@@ -302,7 +369,7 @@ void Playlist::saveAs()
 void Playlist::updateDeletedItem(PlaylistItem *item)
 {
     m_members.remove(item->file().absFilePath());
-
+    m_randomSequence.removeAll(item);
     m_history.removeAll(item);
 }
 
@@ -385,8 +452,6 @@ void Playlist::setSearch(PlaylistSearch* s)
     for(int row = 0; row < topLevelItemCount(); ++row)
         topLevelItem(row)->setHidden(true);
     setItemsVisible(s->matchedItems(), true);
-
-    TrackSequenceManager::instance()->iterator()->playlistChanged();
 }
 
 void Playlist::setSearchEnabled(bool enabled)
@@ -421,7 +486,6 @@ void Playlist::synchronizePlayingItems(Playlist *playlist, bool setMaster)
         PlaylistItem *item = static_cast<PlaylistItem *>(*itemIt);
         if(base == item->collectionItem()) {
             item->setPlaying(true, setMaster);
-            TrackSequenceManager::instance()->setCurrent(item);
             return;
         }
     }
@@ -508,6 +572,20 @@ void Playlist::slotRenameFile()
     emit signalEnableDirWatch(true);
 }
 
+void Playlist::slotBeginPlayback()
+{
+    QTreeWidgetItemIterator visible(this, QTreeWidgetItemIterator::NotHidden);
+    PlaylistItem *item = static_cast<PlaylistItem *>(*visible);
+
+    if(item) {
+        refillRandomList();
+        playNext();
+    }
+    else {
+        action("stop")->trigger();
+    }
+}
+
 void Playlist::slotViewCover()
 {
     const PlaylistItemList items = selectedItems();
@@ -574,7 +652,7 @@ void Playlist::slotAddCover(bool retrieveLocal)
 // Called when image fetcher has added a new cover.
 void Playlist::slotCoverChanged(int coverId)
 {
-    qCDebug(JUK_LOG) << "Refreshing information for newly changed covers.\n";
+    qCDebug(JUK_LOG) << "Refreshing information for newly changed covers.";
     refreshAlbums(selectedItems(), coverId);
 }
 
@@ -614,6 +692,48 @@ void Playlist::slotReload()
     loadFile(m_fileName, fileInfo);
 }
 
+void Playlist::refillRandomList()
+{
+    qCDebug(JUK_LOG) << "Refilling random items.";
+
+    if(action("disableRandomPlay")->isChecked()) {
+        m_randomSequence.clear();
+        return;
+    }
+
+    PlaylistItemList randomItems = visibleItems();
+
+    // See https://www.pcg-random.org/posts/cpp-seeding-surprises.html
+    std::random_device rdev;
+    uint64_t rseed = (uint64_t(rdev()) << 32) | rdev();
+    std::linear_congruential_engine<
+        uint64_t, 6364136223846793005U, 1442695040888963407U, 0U
+        > knuth_lcg(rseed);
+
+    std::shuffle(randomItems.begin(), randomItems.end(), knuth_lcg);
+
+    if(action("albumRandomPlay")->isChecked()) {
+        std::sort(randomItems.begin(), randomItems.end(),
+            [](PlaylistItem *a, PlaylistItem *b) {
+                return a->file().tag()->album() < b->file().tag()->album();
+            });
+
+        // If there is an item playing from our playlist already, move its
+        // album to the front
+
+        const auto wasPlaying = playingItem();
+        if(wasPlaying && wasPlaying->playlist() == this) {
+            const auto playingAlbum = wasPlaying->file().tag()->album();
+            std::stable_partition(randomItems.begin(), randomItems.end(),
+                [playingAlbum](const PlaylistItem *item) {
+                    return item->file().tag()->album() == playingAlbum;
+                });
+        }
+    }
+
+    std::swap(m_randomSequence, randomItems);
+}
+
 void Playlist::slotWeightDirty(int column)
 {
     if(column < 0) {
@@ -724,10 +844,23 @@ void Playlist::removeFromDisk(const PlaylistItemList &items)
 void Playlist::synchronizeItemsTo(const PlaylistItemList &itemList)
 {
     // direct call to ::items to avoid infinite loop, bug 402355
+    m_randomSequence.clear();
     clearItems(Playlist::items());
     createItems(itemList);
 }
 
+void Playlist::beginPlayingItem(PlaylistItem *itemToPlay)
+{
+    if(itemToPlay) {
+        setPlaying(itemToPlay, true);
+        m_collection->requestPlaybackFor(itemToPlay->file());
+    }
+    else {
+        setPlaying(nullptr);
+        action("stop")->trigger();
+    }
+}
+
 void Playlist::dragEnterEvent(QDragEnterEvent *e)
 {
     if(CoverDrag::isCover(e->mimeData())) {
@@ -949,6 +1082,7 @@ void Playlist::read(QDataStream &s)
     s >> files;
 
     QTreeWidgetItem *after = 0;
+    const CollectionList *clInst = CollectionList::instance();
 
     m_blockDataChanged = true;
 
@@ -956,7 +1090,15 @@ void Playlist::read(QDataStream &s)
         if(file.isEmpty())
             throw BICStreamException();
 
-        after = createItem(FileHandle(file), after);
+        const auto &cItem = clInst->lookup(file);
+        if(cItem) {
+            // Reuse FileHandle so the playlist doesn't force TagLib to read it
+            // from disk
+            after = createItem(cItem->file(), after);
+        }
+        else {
+            after = createItem(FileHandle(file), after);
+        }
     }
 
     m_blockDataChanged = false;
@@ -1359,15 +1501,15 @@ void Playlist::slotPopulateBackMenu() const
     }
 }
 
-void Playlist::slotPlayFromBackMenu(QAction *backAction) const
+void Playlist::slotPlayFromBackMenu(QAction *backAction)
 {
     int number = backAction->data().toInt();
 
     if(number >= m_backMenuItems.size())
         return;
 
-    TrackSequenceManager::instance()->setNextItem(m_backMenuItems[number]);
-    action("forward")->trigger();
+    auto &nextItem = m_backMenuItems[number];
+    beginPlayingItem(nextItem);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -1459,33 +1601,28 @@ void Playlist::loadFile(const QString &fileName, const QFileInfo &fileInfo)
     playlistItemsChanged();
 }
 
+// static
 void Playlist::setPlaying(PlaylistItem *item, bool addToHistory)
 {
-    if(playingItem() == item)
+    auto wasPlayingItem = playingItem();
+    if(wasPlayingItem == item)
         return;
 
-    if(playingItem()) {
-        if(addToHistory) {
-            if(playingItem()->playlist() ==
-               playingItem()->playlist()->m_collection->upcomingPlaylist())
-                m_history.append(playingItem()->collectionItem());
-            else
-                m_history.append(playingItem());
-        }
-        playingItem()->setPlaying(false);
+    if(wasPlayingItem && addToHistory) {
+        m_history.append(wasPlayingItem->collectionItem());
+
+        const bool enableBack = !m_history.isEmpty();
+        action<KToolBarPopupAction>("back")->menu()->setEnabled(enableBack);
     }
 
-    TrackSequenceManager::instance()->setCurrent(item);
-    // TODO is this replaced by MPRIS2?
-    //kapp->dcopClient()->emitDCOPSignal("Player", "trackChanged()", data);
+    if(wasPlayingItem) {
+        // NB: will clear the whole list of playing items recursively
+        wasPlayingItem->setPlaying(false);
+    }
 
-    if(!item)
-        return;
-
-    item->setPlaying(true);
-
-    bool enableBack = !m_history.isEmpty();
-    action<KToolBarPopupAction>("back")->menu()->setEnabled(enableBack);
+    if(item) {
+        item->setPlaying(true);
+    }
 }
 
 bool Playlist::playing() const
@@ -1636,11 +1773,12 @@ void Playlist::cleanupAfterAllFileLoadsCompleted()
     // and let user adjust from there.
     if(manualResize()) {
         auto manualResizeAction = action<KToggleAction>("resizeColumnsManually");
+        auto wasChecked = manualResizeAction->isChecked();
 
-        manualResizeAction->toggle();
+        manualResizeAction->setChecked(false);
         calculateColumnWeights();
         slotUpdateColumnWidths();
-        manualResizeAction->toggle();
+        manualResizeAction->setChecked(wasChecked);
     }
 
     playlistItemsChanged();
@@ -1658,12 +1796,16 @@ void Playlist::slotUpdateColumnWidths()
     // Make sure that the column weights have been initialized before trying to
     // update the columns.
 
-    QList<int> visibleColumns;
+    QVector<int> visibleColumns;
     for(int i = 0; i < columnCount(); i++) {
         if(!isColumnHidden(i))
             visibleColumns.append(i);
     }
 
+    if(visibleColumns.isEmpty()) {
+        return;
+    }
+
     // convenience handler for deprecated text metrics
     const auto textWidth = [](const QFontMetrics &fm, const QString &text) {
 #if (QT_VERSION >= QT_VERSION_CHECK(5, 11, 0))
@@ -1725,6 +1867,11 @@ void Playlist::slotUpdateColumnWidths()
     foreach(int column, visibleColumns)
         totalWeight += m_columnWeights[column];
 
+    // This can happen during startup, before we have the tracks loaded.
+    if(qFuzzyIsNull(totalWeight)) {
+        return;
+    }
+
     // Computed a "weighted width" for each visible column.  This would be the
     // width if we didn't have to handle the cases of minimum and maximum widths.
 
@@ -2044,8 +2191,8 @@ void Playlist::slotPlayCurrent()
 {
     QTreeWidgetItemIterator it(this, QTreeWidgetItemIterator::Selected);
     PlaylistItem *next = static_cast<PlaylistItem *>(*it);
-    TrackSequenceManager::instance()->setNextItem(next);
-    action("forward")->trigger();
+    beginPlayingItem(next);
+    refillRandomList();
 }
 
 void Playlist::slotUpdateTime()
