@@ -1,6 +1,6 @@
 /**
  * Copyright (C) 2004 Scott Wheeler <wheeler@kde.org>
- * Copyright (C) 2009 Michael Pyne <mpyne@kde.org>
+ * Copyright (C) 2009, 2021 Michael Pyne <mpyne@kde.org>
  *
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
@@ -52,7 +52,6 @@
 #include "directorylist.h"
 #include "mediafiles.h"
 #include "playermanager.h"
-#include "tracksequencemanager.h"
 #include "juk.h"
 
 //Laurent: readd it
@@ -212,20 +211,17 @@ void PlaylistCollection::createDynamicPlaylist(const PlaylistList &playlists)
 
 void PlaylistCollection::showMore(const QString &artist, const QString &album)
 {
+    if(showMoreActive()) {
+        clearShowMore();
+    }
 
     PlaylistList playlists;
     PlaylistSearch::ComponentList components;
 
-    if(currentPlaylist() != CollectionList::instance() &&
-       currentPlaylist() != m_showMorePlaylist)
-    {
-        playlists.append(currentPlaylist());
-    }
+    playlists.append(visiblePlaylist());
 
-    playlists.append(CollectionList::instance());
-
-    if(!artist.isNull())
-    { // Just setting off the artist stuff in its own block.
+    if(!artist.isEmpty()) {
+        // Just setting off the artist stuff in its own block.
         ColumnList columns;
         columns.append(PlaylistItem::ArtistColumn);
         PlaylistSearch::Component c(artist, false, columns,
@@ -233,7 +229,7 @@ void PlaylistCollection::showMore(const QString &artist, const QString &album)
         components.append(c);
     }
 
-    if(!album.isNull()) {
+    if(!album.isEmpty()) {
         ColumnList columns;
         columns.append(PlaylistItem::AlbumColumn);
         PlaylistSearch::Component c(album, false, columns,
@@ -645,7 +641,7 @@ Playlist *PlaylistCollection::currentPlaylist() const
     if(m_belowDistraction)
         return m_belowDistraction;
 
-    if(m_upcomingPlaylist && m_upcomingPlaylist->active())
+    if(m_upcomingPlaylist && m_upcomingPlaylist->topLevelItemCount() > 0)
         return m_upcomingPlaylist;
 
     if(Playlist::playingItem())
@@ -666,7 +662,6 @@ void PlaylistCollection::raise(Playlist *playlist)
     if(m_dynamicPlaylist && currentPlaylist() == m_dynamicPlaylist)
         m_dynamicPlaylist->lower(playlist);
 
-    TrackSequenceManager::instance()->setCurrentPlaylist(playlist);
     playlist->applySharedSettings();
     playlist->setSearchEnabled(m_searchEnabled);
     m_playlistStack->setCurrentWidget(playlist);
@@ -716,6 +711,12 @@ void PlaylistCollection::clearShowMore(bool raisePlaylist)
     if(!m_showMorePlaylist)
         return;
 
+    const auto playingItem = currentPlaylist()->playingItem();
+    if(playingItem && playingItem->playlist() == m_showMorePlaylist) {
+        // Reset the playing indicator to the corresponding CollectionList item
+        playingItem->collectionItem()->setPlaying(true, true);
+    }
+
     if(raisePlaylist) {
         if(m_belowShowMorePlaylist)
             raise(m_belowShowMorePlaylist);
@@ -723,7 +724,7 @@ void PlaylistCollection::clearShowMore(bool raisePlaylist)
             raise(CollectionList::instance());
     }
 
-    m_belowShowMorePlaylist = 0;
+    m_belowShowMorePlaylist = nullptr;
 }
 
 void PlaylistCollection::enableDirWatch(bool enable)
